/*
 *
 * See attached license Agreement.
 *
 * IBM MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 *
 */

import java.io.*;
import java.util.*;
import java.sql.*;
import javax.servlet.*;
import javax.servlet.http.*;
import imaginary.sql.*;

public class DbDemoServlet extends HttpServlet
{
    // START_OF_COPYRIGHT
    // (do not edit this message, as it is generated automatically)
    public static String copyright="(C) Copyright IBM Corp. 1996, 1997, All rights reserved.";
    // END_OF_COPYRIGHT


    public void service(HttpServletRequest req,
                        HttpServletResponse res) throws ServletException, IOException
    {
        res.setContentType("text/html");
        ServletOutputStream out = res.getOutputStream();
        out.print("<HTML><BODY>\n");
        out.print("<H1> Data base demo </H1>\n");

        try
        {
            Hashtable h = getFormParameters(req);

            dbAccess(out, h, getServletConfig().getInitParameter("host"));
        }
        catch(Exception e)
        {
            e.printStackTrace();
            out.print("Exception :" + e.toString());
        }

        out.print("</HTML></BODY>\n");
    }

    public String getServletInfo()
    {
        return "Demonstrates data base access ...";
    }


    private Hashtable getFormParameters(HttpServletRequest req) throws ServletException, IOException
    {
        String method = req.getMethod();
        String query = null;

        if(null != method)
        {
            if(0 == method.compareTo("GET"))
            {
                query = req.getQueryString();
            }
            else if(0 == method.compareTo("POST"))
            {
                int contentLength = req.getContentLength();
                if(contentLength > 0)
                {
                    ServletInputStream in = req.getInputStream();
                    byte []b = new byte[contentLength];
                    int thisTime;
                    for(int soFar = 0; soFar < b.length; soFar += thisTime)
                    {
                        thisTime = in.read(b, soFar, b.length - soFar);
                        if (thisTime == -1)
                            throw new ServletException(" Content len is wrong or client disconnected ...");
                    }
                    query = new String(b, 0);
                }
            }
        }

        if(null == query)
        {
            throw new ServletException("bad method or query");
        }

        return HttpUtils.parseQueryString(query);
    }

    private void dbAccess(ServletOutputStream out,
                          Hashtable h,
                          String host) throws IOException
    {
	    String stack        = (String)h.get("stack");
	    String username     = (String)h.get("username");
	    String password     = (String)h.get("password");
	    String query        = (String)h.get("query");
	    String operation    = (String)h.get("operation");

	    Connection con;

	    if (stack == null   ||
	        username == null||
	        password == null||
	        query == null   ||
	        operation == null)
	    {
	        out.println("Error some of the needed information is missing ...");
    	    return;
	    }

	    String url   = "jdbc:msql://"+ host + "/" + stack;
	    out.println("<hr><h3>Previous Query</h3>");
	    out.println("<pre>");
	    out.println("Database Stack : "+stack + "\r\n");
	    out.println("       User ID : "+username + "\r\n");
    	out.println("      Password : "+password + "\r\n");
	    out.println("         Query : "+query + "\r\n");
	    out.println("           url : "+url + "\r\n");
	    out.println("</pre>");

	    try
	    {
	        Class.forName("imaginary.sql.iMsqlDriver");

	        // get the connection to the database
	        con = DriverManager.getConnection(url, username, password);

	        out.println("<hr>");
	        out.println("<h3>Driver and Database Messages</h3>");
	        checkForWarning(con.getWarnings(), out);
	        DatabaseMetaData dma = con.getMetaData();
            if(null != dma)
            {
    	        out.println("Connected to " + dma.getURL() + "<br>");
	            out.println("Driver       " + dma.getDriverName() + "<br>");
	            out.println("Version      " + dma.getDriverVersion() + "<br>");
	        }
	        Statement stmt = con.createStatement();
	        if(0 == operation.compareTo("query"))
	        {
    	        // execute the query
	            ResultSet rs = stmt.executeQuery(query);
	            // print out the result
    	        dispResultSet(rs, out);
	            rs.close();
    	    }
            else if(0 == operation.compareTo("update"))
            {
                out.println(stmt.executeUpdate(query) + " were updated ...");
            }
            else if(0 == operation.compareTo("execute"))
            {
                out.println("Execution " + (stmt.execute(query) ? "was O.K " : " failed "));
            }
            else
            {
                out.println("Bad operation ...");
            }
	        stmt.close();
	        con.close();
	        out.println("<hr>");
	    }
	    catch (SQLException ex)
	    {
	        out.println("<hr>*** SQLException caught ***<p>");
	        while (ex != null)
	        {
	            out.println("SQLState: " + ex.getSQLState() + "<br>");
    	        out.println("Message:  " + ex.getMessage() + "<br>");
	            out.println("Vendor:   " + ex.getErrorCode() + "<br>");
	            ex = ex.getNextException();
	        }
	    }
	    catch (java.lang.Exception ex)
	    {
	        out.println("<hr>*** Exception caught ***<p>");
	    }
    }

    private void checkForWarning(SQLWarning warn, ServletOutputStream out) throws SQLException, IOException
    {
       boolean rc = false;

       if(warn != null)
       {
          out.println("<hr>*** Warning ***<p>");
          rc = true;
          while (warn != null)
          {
             out.println("SQLState: " + warn.getSQLState() + "<br>");
             out.println("Message:  " + warn.getMessage() + "<br>");
             out.println("Vendor:   " + warn.getErrorCode() + "<br>");
             warn = warn.getNextWarning();
          }
       }
       else
       {
        out.println("*** None ***<p>");
       }
    }

    /*
     * display the result in html table format
     */
    private void dispResultSet(ResultSet rs, ServletOutputStream out) throws SQLException, IOException
    {
       int i;

       if(null == rs)
        return;

       // metadata can supply information about the schema
       ResultSetMetaData rsmd = rs.getMetaData();
       int numCols = rsmd.getColumnCount();
       out.println("<hr>");
       out.println("<h3>Database Columns and Data</h3>");
       out.println("<table border=3>");
       out.println("<tr>");
       for (i=1; i<=numCols; i++)
       {
          out.println("<th>" + rsmd.getColumnLabel(i) + "</th>");
       }
       out.println("</tr>");

       // for entire data
       while (rs.next())
       {
    	    out.println("<tr>");

    	    // for one row
            for (i=1; i<=numCols; i++)
            {
	            dispElement(rs, rsmd.getColumnType(i), out, i);
            }
	        out.println("</tr>");
       }
       out.println("</table>");
    }


    // print one element
    private void dispElement(ResultSet rs,
                            int dataType,
			                ServletOutputStream out,
			                int col) throws SQLException, IOException
    {

        // ask for data depending on the datatype
	    switch(dataType)
	    {
	    case Types.DATE:
	        java.sql.Date date = rs.getDate(col);
	        out.print("<th>" + date.toString() + "</th>");
	    break;
	    case Types.TIME:
	        java.sql.Time time = rs.getTime(col);
	        out.print("<th>" + time.toString() + "</th>");
	    break;
	    case Types.TIMESTAMP:
    	  java.sql.Timestamp timestamp = rs.getTimestamp(col);
	      out.print("<th>" + timestamp.toString() + "</th>");
    	break;
	    case Types.CHAR:
    	case Types.VARCHAR:
	    case Types.LONGVARCHAR:
    	  String str = rs.getString(col);
	      out.print("<th>" + str + "</th>");
        break;
	    case Types.BIT:
	        boolean bit = rs.getBoolean(col);
	        out.print("<th>" + new Boolean(bit) + "</th>");
	    break;
	    case Types.TINYINT:
	        byte tinyint = rs.getByte(col);
	        out.print("<th>" + new Integer(tinyint) + "</th>");
	    break;
	    case Types.SMALLINT:
	        short smallint = rs.getShort(col);
	        out.print("<th>" + new Integer(smallint) + "</th>");
	    break;
	    case Types.INTEGER:
	        int integer = rs.getInt(col);
	        out.print("<th>" + new Integer(integer) + "</th>");
	    break;
	    case Types.BIGINT:
	        long bigint = rs.getLong(col);
	        out.print("<th>" + new Long(bigint) + "</th>");
	    break;
	    case Types.REAL:
	        float real = rs.getFloat(col);
	        out.print("<th>" + new Float(real) + "</th>");
	    break;
    	case Types.FLOAT:
	    case Types.DOUBLE:
	        double longreal = rs.getDouble(col);
	        out.print("<th>" + new Double(longreal) + "</th>");
	    break;
	    case Types.BINARY:
	    case Types.VARBINARY:
	    case Types.LONGVARBINARY:
	        byte[] binary = rs.getBytes(col);
	        out.print("<th>" + new String(binary, 0) + "</th>");
	    break;
	    }
    }
};
