/*
 *
 * See attached license Agreement.
 *
 * IBM MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 *
 */

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import java.util.Date;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 * A very simple servlet to demo a file handler.
 */
public class MFileServlet extends HttpServlet
{
    // START_OF_COPYRIGHT
    // (do not edit this message, as it is generated automatically)
    public static String copyright="(C) Copyright IBM Corp. 1996, 1997, All rights reserved.";
    // END_OF_COPYRIGHT

    public static final String M_DEFAULT_FILE_NAME  = "index.html";
    public static final String M_DEFAULT_FILE_TAG   = "default.file";

    private ServletContext  m_ctxt;
    private String          m_defaultFile = null;

    public void init(ServletConfig cfg) throws ServletException
    {
        super.init(cfg);
        m_defaultFile = cfg.getInitParameter(M_DEFAULT_FILE_TAG);
        if(null == m_defaultFile)
        {
            m_defaultFile = M_DEFAULT_FILE_NAME;
        }
        m_ctxt = getServletContext();
    }

    /**
     * Returns the file pointed by the uri.
     *
     *
     */
    public void service(HttpServletRequest req, HttpServletResponse res)
        throws ServletException
    {
        try
        {
            /*
             * get the uri and check if it's evil (see util.h in nsapi).
             * Check that it has no ../ or //.
             */
            String uri = req.getRequestURI();
            if((uri.indexOf("..") != -1) || (uri.indexOf("//") != -1))
            {
                System.out.println("HttpServletResponse.SC_FORBIDDEN " + uri);
                res.sendError(HttpServletResponse.SC_FORBIDDEN, "Don't mess with illigal uri's PLEASE");
                return;
            }

            /*
             * This servlet serves only get and head (file servlet isn't it ?).
             * So we will take the method and check it. It should be noted that
             * we could also serve put/post etc' so we will return SC_NOT_IMPLEMENTED.
             */
            String method = req.getMethod();

            if((0 != method.compareTo("GET")) && (0 != method.compareTo("HEAD")))
            {
                System.out.println("sendError(HttpServletResponse.SC_NOT_IMPLEMENTED " + method);
                res.sendError(HttpServletResponse.SC_NOT_IMPLEMENTED);
                return;
            }

            String realPath = m_ctxt.getRealPath(uri);
            if(null == realPath)
            {
                // internal error !!!
                System.out.println("sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR " + uri);
                res.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
                return;
            }

            File file = new File(realPath);
            if(file.isDirectory())
            {
                if(realPath.endsWith(File.separator))
                {
                    realPath = realPath + m_defaultFile;
                }
                else
                {
                        realPath = realPath + File.separator + m_defaultFile;
                }
                file = new File(realPath);
            }

            if(!file.exists() || file.isDirectory())
            {
                System.out.println("HttpServletResponse.SC_MOVED_PERMANENTLY " + file);
                res.sendError(HttpServletResponse.SC_MOVED_PERMANENTLY);
                return;
            }

            Date d = new Date();
            System.out.println("Date            : " + d.toGMTString());
            System.out.println("Serving file    : " + file);
            System.out.println("To host         : " + req.getRemoteHost());
            System.out.flush();

            /*
             * I'm lazy so I'll not check the accept header with
             * the mime type
             */
            long fileLength = file.length();
            String fileMime = m_ctxt.getMimeType(realPath);

            res.setStatus(HttpServletResponse.SC_OK);
            res.setContentLength((int)fileLength);
            if(null != fileMime)
            {
                res.setContentType(fileMime);
            }
            else
            {
                res.setContentType("application/octet-stream");
            }
            res.setDateHeader("Last-Modified", file.lastModified());

            /*
             * If the method is get, lets send the file ...
             */
            if(0 == method.compareTo("GET"))
            {
                ServletOutputStream out = res.getOutputStream();
                FileInputStream fileInputStream = new FileInputStream(file);
                int n;
                byte b[] = new byte[2048];
                for(n = 0 ; fileLength > 0 ; fileLength -= n)
                {
                    n = fileInputStream.read(b, 0, b.length);
                    if(n <= 0)
                    {
                        // done ...
                        break;
                    }
                    out.write(b, 0, n);
                }
                out.flush();
                fileInputStream.close();
            }
        }
        catch(IOException ioe)
        {
            log("File servlet, error : " + ioe.toString());
            ioe.printStackTrace();
        }
    }

    public String getServletInfo()
    {
        return "This Servlet is a demo file serving servlet.";
    }
}
